/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozCmgui.
 *
 * The Initial Developer of the Original Code is
 * Auckland Uniservices Ltd, Auckland, New Zealand.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
#include "zincDownloadMonitorSrc.h"
#include "nsIRunnable.h"
#include <nsCOMPtr.h>
#include <prmem.h>

NS_IMPL_ISUPPORTS1(zincDownloadMonitor, zincIDownloadMonitor)

zincDownloadMonitor::zincDownloadMonitor()
  : mCount(0)
{
}

NS_IMETHODIMP
zincDownloadMonitor::GetCount(PRUint32* ret)
{
  *ret = mCount;
  return NS_OK;
}

NS_IMETHODIMP
zincDownloadMonitor::IncrementCounter()
{
  mCount++;
  return NS_OK;
}

NS_IMETHODIMP
zincDownloadMonitor::DecrementCounter()
{
  mCount--;
  if (mCount == 0 && mOnDownloadsCompleted != nsnull)
    mOnDownloadsCompleted->Run();
  return NS_OK;
}

NS_IMETHODIMP
zincDownloadMonitor::GetOnDownloadsCompleted(nsIRunnable** ret)
{
  *ret = mOnDownloadsCompleted;
  if (mOnDownloadsCompleted != nsnull)
    NS_ADDREF(*ret);
  return NS_OK;
}

NS_IMETHODIMP
zincDownloadMonitor::SetOnDownloadsCompleted(nsIRunnable* r)
{
  mOnDownloadsCompleted = r;
  return NS_OK;
}

NS_IMETHODIMP
zincDownloadMonitor::GetCommandData(CmguiICommandData** ret)
{
  *ret = mCommandData;
  if (mCommandData != nsnull)
    NS_ADDREF(*ret);
  return NS_OK;
}

NS_IMETHODIMP
zincDownloadMonitor::SetCommandData(CmguiICommandData* r)
{
  mCommandData = r;
  return NS_OK;
}

NS_IMETHODIMP
zincDownloadMonitor::DefineMemoryBlockFromUrl(
	const nsACString& file,
	const nsACString& memoryName)
{
  nsresult rv;
  nsCOMPtr<nsIURI> uri;

  rv = NS_NewURI(getter_AddRefs(uri), file, nsnull, nsnull, nsnull);
  if ( NS_SUCCEEDED( rv ) ) {
    //printf( "...NS_NewURI completed OK\n" );
  } else {
    printf( "%s %d: NS_NewURI failed, rv=0x%08X\n", (char*)__FILE__, (int)__LINE__, (int)rv );
    return rv;
  }

  IncrementCounter();

  DnloadListener *listener = new DnloadListener(memoryName, this);

  if (listener == nsnull)
    return NS_ERROR_OUT_OF_MEMORY;
  NS_ADDREF(listener);

  rv = NS_OpenURI(listener, nsnull , uri, nsnull, nsnull, nsnull);

  NS_RELEASE(listener);

  if ( NS_SUCCEEDED( rv ) ) {
    //printf( "...NS_OpenURI completed OK\n" );
  } else {
    printf( "%s %d: NS_OpenURI failed, rv=0x%08X\n", (char*)__FILE__, (int)__LINE__, (int)rv );
    return rv;
  }

  return NS_OK;
}



DnloadListener::DnloadListener(const nsACString& aMemoryName, 
	zincIDownloadMonitor* aDownloadMonitor)
	: mDownloadMonitor(aDownloadMonitor), mMemoryName(aMemoryName)
{
	mMemory = NULL;
}

DnloadListener::~DnloadListener()
{
	if (mMemory)
	{
		//We can't free it yet
		//PR_Free(mMemory);
	}
}


NS_IMPL_ISUPPORTS2(DnloadListener,
                   nsIRequestObserver,
                   nsIStreamListener);

NS_IMETHODIMP
DnloadListener::OnStartRequest(nsIRequest* request, nsISupports* context)
{
  //printf( "DnloadListener::OnStartRequest entered\n");

  mReadChunk=10000;

  mMemory = PR_Malloc(mReadChunk*sizeof(char));
  if (!mMemory)
  {
    printf( "%s %d: Malloc error\n",(char*)__FILE__, (int)__LINE__);
  }
  mMemoryLength = mReadChunk;
  mTotalRead=0;

  //printf( "DnloadListener::OnStartRequest exited\n");
  return NS_OK;
}



NS_IMETHODIMP
DnloadListener::OnDataAvailable(nsIRequest* request,
                              nsISupports* context,
                              nsIInputStream *aStream, 
                              PRUint32 offset,
                              PRUint32 aLength)
{
  int return_code;
  PRUint32 bytesRead;
  void  *new_data;
  char *stream_start;
  nsresult rv = NS_OK;

  //printf("DnloadListener::OnDataAvailable [offset=%u length=%u]\n",offset, aLength);
  
  return_code = 1;
  while ( return_code ) 
  {
    if (mTotalRead + mReadChunk > mMemoryLength)
    {
      if ((new_data = (char *)PR_Realloc((void *)mMemory, (mMemoryLength + mReadChunk))))
      {
        mMemory = new_data;
        mMemoryLength += mReadChunk;
      }
      else
      {
        printf("%s %d: Realloc error, rv=0x%08X\n",(char*)__FILE__,
               (int)__LINE__, (int)rv);
        return_code = 0;
      }
    }
    if (return_code)
    {
      stream_start = (char *)mMemory + mTotalRead;
      rv = aStream->Read(stream_start, mReadChunk, &bytesRead );
      if ( NS_SUCCEEDED( rv ) ) 
      {
        mTotalRead += bytesRead;
        if ( bytesRead < mReadChunk)
	{
          return_code = 0;
	} 
      } 
      else 
      {
        printf( "%s %d: Read error, rv=0x%08X\n",(char*)__FILE__, (int)__LINE__, (int)rv );
        return_code = 0;
      }
    }
  }

  if (mMemoryLength != mTotalRead)
  {
    if ((new_data = PR_Realloc(mMemory, mTotalRead*sizeof(char))))
      {
        mMemory = new_data;
        mMemoryLength = mTotalRead;
      }
      else
      {
        printf( "%s %d: Realloc error, rv=0x%08X\n",(char*)__FILE__, (int)__LINE__, (int)rv );
        return_code = 0;
      }
  }

  //printf( "DnloadListener::OnDataAvailable exit \n");
  return NS_OK;
}


NS_IMETHODIMP
DnloadListener::OnStopRequest(nsIRequest* request, nsISupports* context,
                            nsresult aStatus)
{
  //printf( "DnloadListener::OnStopRequest\n");
  
  //puts((char *)mMemory);
  char *name = ToNewCString(mMemoryName);
  if (mDownloadMonitor != nsnull)
  {
	  nsCOMPtr<CmguiICommandData> commandData;
	  mDownloadMonitor->GetCommandData(getter_AddRefs(commandData));
	  commandData->DefineMemoryBlock(name, NS_REINTERPRET_CAST(PRUint8*,mMemory), mMemoryLength);
	  mDownloadMonitor->DecrementCounter();
  }

  mDownloadMonitor = nsnull;

  return NS_OK;
}

